import time
import json
import os
import math
import multiprocessing
import tempfile
import random
import shutil
import socket
import struct
from concurrent.futures import ProcessPoolExecutor
from multiprocessing import Manager
from rich.progress import Progress
from rich.console import Console
from mando import command, arg

from ketacli.sdk.util import Template, format_bytes
from ketacli.sdk.base.client import request_post

from ketacli.mock.log_generator import LogGenerator

# 尝试导入ujson，如果不可用则使用标准json
try:
    import ujson as json_serializer
    USE_UJSON = True
except ImportError:
    import json as json_serializer
    USE_UJSON = False

# 创建控制台对象
console = Console()

 # 使用LogGenerator类生成日志
        
        
# 创建日志生成器实例
log_generator = LogGenerator()


def process_batch_data(batch_data, query_params, gzip, progress, task_id):
    """
    Process a batch of pre-loaded data and upload them to the server.
    :param batch_data: List of data lines to process.
    :param query_params: Query parameters for the upload.
    :param gzip: Whether to use gzip for the request.
    :param progress: Shared progress object.
    :param task_id: Task ID for tracking progress.
    :return: Tuple of data length and response.
    """
    if not batch_data:
        progress[task_id] += len(batch_data) if batch_data else 0
        return 0, None
    
    data_length = 0
    local_datas = []
    
    # 合并处理：遍历批量数据，同时进行JSON解析和模板渲染
    try:
        for line in batch_data:
            if not line:  # 确保行不为空
                continue
            data_length += len(line)
            try:
                local_datas.append(json.loads(line))
            except Exception as e:
                print(f"Error parsing line: {line}, error: {str(e)}")
                # 跳过错误的行，继续处理
                continue
                
    except Exception as e:
        print(f"Error processing batch data: {str(e)}")
        progress[task_id] += len(batch_data)
        return 0, None
    
    # 如果没有解析到有效数据，直接返回
    if not local_datas:
        progress[task_id] += len(batch_data)
        return 0, None
    
    # 发送到服务端
    response = None
    if local_datas:
        response = request_post("data", local_datas, query_params, gzip=gzip).json()
    
    # 更新进度条
    progress[task_id] += len(batch_data)
    return data_length, response


def generate_and_upload(data, count, query_params, gzip, progress, task_id, output_type='server', output_file=None, worker_id=None, render=True):
    """
    Generate mock data and upload in a batch.
    :param data: The JSON string template.
    :param count: Number of data items to generate.
    :param query_params: Query parameters for the upload.
    :param gzip: Whether to use gzip for the request.
    :param progress: Shared progress object.
    :param task_id: Task ID for tracking progress.
    :param output_type: Where to write the data, 'server' or 'file'.
    :param output_file: File path to write data when output_type is 'file'.
    :param worker_id: Worker ID for creating worker-specific temp files.
    :return: Tuple of data length and response.
    """
    # # 创建一次Template对象，避免重复创建
    temp = Template(data)
    
    # # 使用批量渲染功能一次性生成所有数据
    rendered_texts = temp.batch_render(count, render=render)
    
    # 直接计算数据长度，避免额外的迭代
    data_length = sum(len(text) for text in rendered_texts)
    
    # 预分配列表大小以避免动态扩展
    local_datas = [None] * count
    
    # 批量解析JSON - 使用分块处理以提高性能
    CHUNK_SIZE = 5000  # 每次处理的数据量
    for i in range(0, count, CHUNK_SIZE):
        chunk = rendered_texts[i:i+CHUNK_SIZE]
        # 使用列表推导式批量解析JSON并直接赋值
        parsed_chunk = [json.loads(text) for text in chunk]
        # 将解析结果放入预分配的列表中
        for j, item in enumerate(parsed_chunk):
            local_datas[i+j] = item

    response = None
    if local_datas:
        if output_type == 'server':
            # 发送到服务端
            response = request_post("data", local_datas, query_params, gzip=gzip).json()
        elif output_type == 'file' and output_file:
            # 确定写入的文件路径
            # 如果是多进程模式，每个进程写入自己的临时文件
            actual_output_file = output_file
            if worker_id is not None:
                # 创建临时文件，使用worker_id作为文件名的一部分
                temp_dir = os.path.dirname(os.path.abspath(output_file))
                file_name = os.path.basename(output_file)
                base_name, ext = os.path.splitext(file_name)
                actual_output_file = os.path.join(temp_dir, f"{base_name}_temp_{worker_id}{ext}")
            
            try:
                # 确保目录存在
                os.makedirs(os.path.dirname(os.path.abspath(actual_output_file)), exist_ok=True)
                
                # 使用更大的缓冲区和批量写入
                with open(actual_output_file, 'a', encoding='utf-8', buffering=32768) as f:
                    json_strings = [json_serializer.dumps(item) for item in local_datas]
                    serialized_batch = '\n'.join(json_strings)
                    # 一次性写入所有数据
                    f.write(serialized_batch)
                    f.write('\n')  # 确保最后一行也有换行符
                response = {"total": data_length, "success": data_length, "failure": 0, "temp_file": actual_output_file}
            except Exception as e:
                response = {"total": data_length, "success": 0, "failure": data_length, "temp_file": actual_output_file}

    # 释放内存
    del rendered_texts
    del local_datas
    
    # 更新进度条
    progress[task_id] += count
    return data_length, response


def handle_generate_to_server(repo, data, number, batch, gzip, workers, render):
    """
    场景1: 生成日志并发送到服务端
    """
    query_params = {"repo": repo}
    
    # 确保workers至少为1
    workers = max(1, workers)
    
    # 计算每个工作进程处理的项目数
    items_per_worker = math.ceil(number / workers)
    
    # 共享进度管理
    manager = Manager()
    progress = manager.dict()
    task_ids = []

    # 创建任务列表，均匀分配任务
    remaining_count = number
    
    # 初始化进度条任务
    with Progress(console=console) as prog:
        task = prog.add_task("Generating and sending to server...", total=number)

        # 使用上下文管理器确保资源正确释放
        with ProcessPoolExecutor(max_workers=workers) as executor:
            futures = []
            
            # 准备所有工作进程的任务并直接按工作进程分组
            tasks_by_worker = {}
            for i in range(workers):
                # 计算当前工作进程的项目数
                worker_count = min(items_per_worker, remaining_count)
                if worker_count <= 0:
                    break
                    
                # 计算当前工作进程的批次数和最后一个批次的大小
                full_batches = worker_count // batch
                last_batch_size = worker_count % batch
                
                # 初始化进度
                task_id = f"worker_{i}"
                task_ids.append(task_id)
                progress[task_id] = 0
                
                # 初始化当前工作进程的任务列表
                tasks_by_worker[i] = []
                
                # 准备完整批次的任务
                for j in range(full_batches):
                    tasks_by_worker[i].append({
                        "worker_id": i,
                        "task_id": task_id,
                        "batch_size": batch,
                        "is_last_batch": False
                    })
                
                # 准备最后一个不完整批次（如果有）
                if last_batch_size > 0:
                    tasks_by_worker[i].append({
                        "worker_id": i,
                        "task_id": task_id,
                        "batch_size": last_batch_size,
                        "is_last_batch": True
                    })
                    
                # 更新剩余项目数
                remaining_count -= worker_count

            # 提交任务
            for worker_id in sorted(tasks_by_worker.keys()):
                for task_info in tasks_by_worker[worker_id]:
                    futures.append(
                        executor.submit(
                            generate_and_upload, 
                            data, 
                            task_info["batch_size"], 
                            query_params, 
                            gzip, 
                            progress, 
                            task_info["task_id"],
                            "server", 
                            None,
                            None,
                            render
                        )
                    )            
            
            return _collect_results(futures, progress, number, prog, task)


def handle_generate_to_file(data, number, batch, workers, output_file, render):
    """
    场景2: 生成日志并写文件
    """
    # 如果是文件输出模式，初始化文件
    try:
        # 确保目录存在
        os.makedirs(os.path.dirname(os.path.abspath(output_file)), exist_ok=True)
        # 如果是单进程模式，直接初始化输出文件
        if workers is None or workers <= 1:
            with open(output_file, 'w', encoding='utf-8', buffering=32768) as f:
                pass  # 只创建/清空文件，不写入内容
    except Exception as e:
        console.print(f"Error creating output file: {str(e)}")
        return None, []
    
    # 确保workers至少为1
    workers = max(1, workers)
    
    # 计算每个工作进程处理的项目数
    items_per_worker = math.ceil(number / workers)
    
    # 共享进度管理
    manager = Manager()
    progress = manager.dict()
    task_ids = []

    # 创建任务列表，均匀分配任务
    remaining_count = number
    
    # 初始化进度条任务
    with Progress(console=console) as prog:
        task = prog.add_task("Generating and writing to file...", total=number)

        # 使用上下文管理器确保资源正确释放
        with ProcessPoolExecutor(max_workers=workers) as executor:
            futures = []
            
            # 准备所有工作进程的任务并直接按工作进程分组
            tasks_by_worker = {}
            for i in range(workers):
                # 计算当前工作进程的项目数
                worker_count = min(items_per_worker, remaining_count)
                if worker_count <= 0:
                    break
                    
                # 计算当前工作进程的批次数和最后一个批次的大小
                full_batches = worker_count // batch
                last_batch_size = worker_count % batch
                
                # 初始化进度
                task_id = f"worker_{i}"
                task_ids.append(task_id)
                progress[task_id] = 0
                
                # 初始化当前工作进程的任务列表
                tasks_by_worker[i] = []
                
                # 准备完整批次的任务
                for j in range(full_batches):
                    tasks_by_worker[i].append({
                        "worker_id": i,
                        "task_id": task_id,
                        "batch_size": batch,
                        "is_last_batch": False
                    })
                
                # 准备最后一个不完整批次（如果有）
                if last_batch_size > 0:
                    tasks_by_worker[i].append({
                        "worker_id": i,
                        "task_id": task_id,
                        "batch_size": last_batch_size,
                        "is_last_batch": True
                    })
                    
                # 更新剩余项目数
                remaining_count -= worker_count

            # 提交任务
            for worker_id in sorted(tasks_by_worker.keys()):
                for task_info in tasks_by_worker[worker_id]:
                    futures.append(
                        executor.submit(
                            generate_and_upload, 
                            data, 
                            task_info["batch_size"], 
                            None,  # 文件模式不需要query_params
                            False,  # 文件模式不需要gzip
                            progress, 
                            task_info["task_id"],
                            "file", 
                            output_file,
                            task_info["worker_id"] if workers > 1 else None,
                            render
                        )
                    )
            return _collect_results(futures, progress, number, prog, task)



def handle_file_to_server(repo, file_path, number, batch, gzip, workers):
    """
    场景3: 读文件并发送到服务端
    """
    query_params = {"repo": repo}
    
    # 预读取文件内容到内存（优化：避免多次文件IO）
    file_lines = []
    try:
        with open(file_path, encoding="utf8") as f:
            for line in f:
                line = line.strip().replace("}{", "}\n{").split("\n")[0]
                if line:
                    file_lines.append(line)
    except Exception as e:
        console.print(f"Error reading file {file_path}: {str(e)}")
        return None, []
    
    total_lines = len(file_lines)
    if total_lines == 0:
        console.print("File is empty")
        return None, []
    
    # 如果没有指定number，则处理文件中的所有行
    if not number:
        number = total_lines
        console.print(f"Processing all {number} lines from file {file_path}")
    elif number <= total_lines:
        console.print(f"Processing {number} lines from file {file_path}")
    else:
        console.print(f"Processing {number} lines from file {file_path} (file has {total_lines} lines, will repeat)")
    
    # 确保workers至少为1
    workers = max(1, workers)
    
    # 计算每个工作进程处理的项目数
    items_per_worker = math.ceil(number / workers)
    
    # 共享进度管理
    manager = Manager()
    progress = manager.dict()
    task_ids = []

    # 创建任务列表，均匀分配任务
    remaining_count = number
    
    # 初始化进度条任务
    with Progress(console=console) as prog:
        task = prog.add_task(f"[green][{workers}]workers[/green]: Reading file and sending to server... ", total=number)

        # 使用上下文管理器确保资源正确释放
        with ProcessPoolExecutor(max_workers=workers) as executor:
            futures = []
            
            # 准备所有工作进程的任务并直接按工作进程分组
            tasks_by_worker = {}
            for i in range(workers):
                # 计算当前工作进程的项目数
                worker_count = min(items_per_worker, remaining_count)
                if worker_count <= 0:
                    break
                    
                # 计算当前工作进程的批次数和最后一个批次的大小
                full_batches = worker_count // batch
                last_batch_size = worker_count % batch
                
                # 初始化进度
                task_id = f"worker_{i}"
                task_ids.append(task_id)
                progress[task_id] = 0
                
                # 初始化当前工作进程的任务列表
                tasks_by_worker[i] = []
                
                # 准备完整批次的任务
                for j in range(full_batches):
                    tasks_by_worker[i].append({
                        "worker_id": i,
                        "task_id": task_id,
                        "batch_size": batch,
                        "is_last_batch": False
                    })
                
                # 准备最后一个不完整批次（如果有）
                if last_batch_size > 0:
                    tasks_by_worker[i].append({
                        "worker_id": i,
                        "task_id": task_id,
                        "batch_size": last_batch_size,
                        "is_last_batch": True
                    })
                    
                # 更新剩余项目数
                remaining_count -= worker_count

            # 提交任务
            for worker_id in sorted(tasks_by_worker.keys()):
                for task_info in tasks_by_worker[worker_id]:
                   
                    
                    # 计算当前批次在文件中的起始行
                    start_line = sum(t["batch_size"] for w in range(worker_id) for t in tasks_by_worker.get(w, []))
                    start_line += sum(t["batch_size"] for t in tasks_by_worker[worker_id] if t["task_id"] == task_info["task_id"] and tasks_by_worker[worker_id].index(t) < tasks_by_worker[worker_id].index(task_info))
                    
                    # 准备当前批次的数据
                    batch_data = []
                    for i in range(task_info["batch_size"]):
                        line_index = (start_line + i) % len(file_lines)
                        batch_data.append(file_lines[line_index])
                    
                    # 提交任务，使用预处理的数据
                    futures.append(
                        executor.submit(
                            process_batch_data,
                            batch_data,
                            query_params,
                            gzip,
                            progress,
                            task_info["task_id"]
                        )
                    )

            return _collect_results(futures, progress, number, prog, task)


def _collect_results(futures, progress, number, prog, task):
    """
    收集任务结果的通用函数
    """
    total_data_length = 0
    responses = []
    temp_files = set()

    # 收集结果并更新进度
    completed = 0
    while completed < number:
        # 计算已完成的项目数
        current_completed = sum(progress.values())
        if current_completed > completed:
            # 更新进度条
            prog.update(task, advance=current_completed - completed)
            completed = current_completed
        
        # 短暂休眠以减少CPU使用
        time.sleep(0.1)
        
        # 检查是否所有任务都已完成
        if all(future.done() for future in futures):
            # 确保进度条显示100%
            final_completed = sum(progress.values())
            if final_completed > completed:
                prog.update(task, advance=final_completed - completed)
            break
    
    # 收集结果
    for future in futures:
        try:
            data_length, resp = future.result()
            total_data_length += data_length
            if resp:
                
                responses.append(resp)
                    
                # 收集临时文件信息
                if isinstance(resp, dict) and 'temp_file' in resp:
                    temp_files.add(resp['temp_file'])
        except Exception as e:
            console.print(f"Task failed: {str(e)}")
    
    return total_data_length, responses, temp_files


@command
def mock_data(repo="default", data=None, file=None, number:int=1, batch:int=2000, gzip=False, workers:int=1, output_type="server", output_file=None, render=False):
    """
    Mock data to specified repo

    :param --repo: The target repo, default: "default"
    :param --data: The json string data default: {"raw":"{{ faker.sentence() }}", "host": "{{ faker.ipv4_private() }}"}
    :param --file: Upload json text from file path.
    :param --number,-n: Number of data, default 1. Use -1 for continuous generation
    :param --worker: for worker process configs like quantity.
    :param --gzip: a boolean for enabling gzip compression.
    :param --batch: to set batch processing size or related configs.
    :param --output_type: Where to write the data, 'server', 'file' or 'stdout', default: 'server'
    :param --output_file: File path to write data when output_type is 'file'
    :param --render: Whether to render the template, default: False. When set to False, it will skip template rendering and use raw text directly, which can improve performance for simple text data.

    
    """
    start = time.time()
    if workers:
        workers = int(workers)
    if batch:
        batch = int(batch)

    # 输入验证
    if repo is None:
        console.print(f"Please specify target repo with --repo")
        return

    if data is None and file is None:
        console.print(f"Please use --data or --file to specify data to upload")
        return

    if file and not os.path.exists(file):
        console.print(f"Error: file {file} does not exist")
        return

    if output_type == 'file' and not output_file:
        console.print("Error: output_file is required when output_type is 'file'")
        return

    if file is not None and output_type != 'server':
        console.print("When using file parameter, output_type can only be 'server'")
        return

    # 根据参数组合确定处理场景并调用对应的处理函数
    try:
        if file:
            # ketacli mock_log --render -n 1000000 --batch 2000  --worker 32 --file test.log --repo benchmark_write_parallel_27_replica_2
            # 场景3: 读文件并发送到服务端
            total_data_length, responses, _ = handle_file_to_server(
                repo, file, number, batch, gzip, workers
            )
        elif output_type == "server":
            # ketacli mock_log --render -n 1000000 --batch 2000  --worker 32 --repo benchmark_write_parallel_27_replica_2
            # 场景1: 生成日志并发送到服务端
            total_data_length, responses, _ = handle_generate_to_server(
                repo, data, number, batch, gzip, workers, render
            )
        elif output_type == "file":
            # 场景2: 生成日志并写文件
            # ketacli mock_log --render -n 1000000 --batch 2000  --worker 32 --repo benchmark_write_parallel_27_replica_2 --output_type file --output_file test.log
            total_data_length, responses, temp_files = handle_generate_to_file(
                data, number, batch, workers, output_file, render
            )
            if workers > 1 and temp_files:
                try:
                    merge_start = time.time()
                    console.print(f"Starting to merge {len(temp_files)} temporary files...")
                    
                    # 合并所有临时文件到最终输出文件
                    with open(output_file, 'w', encoding='utf-8', buffering=32768) as outfile:
                        for temp_file in sorted(temp_files):
                            if os.path.exists(temp_file):
                                with open(temp_file, 'r', encoding='utf-8', buffering=32768) as infile:
                                    shutil.copyfileobj(infile, outfile, 1024*1024)  # 1MB块大小
                            # 删除临时文件
                            os.remove(temp_file)
                
                    merge_duration = time.time() - merge_start
                    console.print(f"Successfully merged {len(temp_files)} temporary files into {output_file} in {merge_duration:.2f} seconds")
                except Exception as e:
                    console.print(f"Error merging files: {str(e)}")
        elif output_type == "stdout":
            # 场景4: 生成日志并输出到标准输出
            total_data_length = 0
            responses = []
            temp_files = []
            
            # 生成并直接输出数据
            if number == -1:
                # 持续生成数据，不设上限
                i = 0
                while True:
                    try:
                        if render:
                            temp = Template(data)
                            rendered_data = temp.render()
                        else:
                            rendered_data = data
                        total_data_length += len(rendered_data)
                        print(rendered_data)
                        i += 1
                    except KeyboardInterrupt:
                        console.print(f"\nStopped by user. Generated {i} records.")
                        break
                    except Exception as e:
                        console.print(f"Error generating data: {str(e)}")
                        break
            else:
                # 生成指定数量的数据
                for i in range(number):
                    try:
                        if render:
                            temp = Template(data)
                            rendered_data = temp.render()
                        else:
                            rendered_data = data
                        total_data_length += len(rendered_data)
                        print(rendered_data)
                    except Exception as e:
                        console.print(f"Error generating data: {str(e)}")
                        break
        else:
            console.print(f"Error: unsupported output_type '{output_type}'")
            return

        # 临时文件合并逻辑已移动到 handle_generate_to_file 方法中

        # 显示结果摘要
        if output_type == "server" or file:
            if responses:
                success_count = sum(r['success'] for r in responses)
                total_count = sum(r['total'] for r in responses)
                console.print(f"Successfully uploaded: {success_count}/{total_count} batches")
            
            
            console.print(f"Total: {format_bytes(total_data_length)}")
        elif output_type == "file":
            console.print(f"Data written to {output_file}")
            console.print(f"Total: {format_bytes(total_data_length)} bytes")
        # stdout输出时不显示文件信息
            
        console.print(f'Total Duration: {time.time() - start:.2f} seconds')
        # 当number为-1时不显示速度统计
        if number != -1:
            console.print(f'速度: {number/(time.time() - start):.2f} 条/s')

    except Exception as e:
        console.print(f"Error during processing: {str(e)}")
        return


@command
@arg("log_type", type=str,
     completer=lambda prefix, **kwd: [x for x in log_generator.get_supported_log_types() if
                                      x.startswith(prefix)])
def mock_log(repo="default", data=None, file=None, number=1, batch=2000, gzip=False, workers=1, output_type="server", output_file=None, render=False, log_type="nginx"):
    """Mock log data to specified repo, with multiple log types support
    :param --repo: The target repo, default: "default"
    :param --data: The json string data default:
        {
            "raw": "{{ faker.sentence(nb_words=10) }}",
            "host": "{{ faker.ipv4_private() }}"
        }
    :param --file: Upload json text from file path.
    :param --number,-n: Number of data, default 1. Use -1 for continuous generation
    :param --output_type: Where to write the data, 'server', 'file' or 'stdout', default: 'server'
    :param --output_file: File path to write data when output_type is 'file'
    :param --render: Whether to render the template, default: False. When set to False, it will skip template rendering and use raw text directly, which can improve performance for simple text data.
    :param --log_type: Type of log to generate, options: 'nginx', 'java', 'linux', default: 'nginx'
    """
    if not data:
       
        
        try:
            # 生成指定类型的日志
            data = log_generator.generate_log(log_type, render)
        except ValueError as e:
            console.print(f"[red]{str(e)}[/red]")
            return
        
    # 直接调用优化后的mock_data函数
    mock_data(repo, data, file, number, batch, gzip, workers, output_type, output_file, render)


@command
def mock_metrics(repo="metrics_keta", data=None, file=None, number=1, batch=2000, gzip=False, workers=1, output_type="server", output_file=None, render=False):
    """Mock metrics data to specified repo
    :param --repo: The target repo, default: "metrics_keta"
    :param --data: The json string data default:
        {
            "host": "{{ faker.ipv4_private() }}",
            "region": "{{ random.choice(['us-west-2', 'ap-shanghai', 'ap-nanjing', 'ap-guangzhou']) }}",
            "os": "{{ random.choice(['Ubuntu', 'Centos', 'Debian', 'TencentOS']) }}",
            "timestamp": {{ int(time.time() * 1000) }},
            "fields": {
                "redis_uptime_in_seconds": {{ random.randint(1,1000000) }},
                "redis_total_connections_received": {{ random.randint(1,1000000) }},
                "redis_expired_keys": {{ random.randint(1,1000000) }}
            }
        }
    :param --file: Upload json text from file path.
    :param --number,-n: Number of data, default 1. Use -1 for continuous generation
    :param --output_type: Where to write the data, 'server', 'file' or 'stdout', default: 'server'
    :param --output_file: File path to write data when output_type is 'file'
    :param --render: Whether to render the template, default: False. When set to False, it will skip template rendering and use raw text directly, which can improve performance for simple text data.

    """
    if not data:
        # 使用更紧凑的JSON格式，减少解析开销
        if render:
            data = (
                '{"host":"{{ faker.ipv4_private() }}",'
                '"region":"{{ random.choice([\"us-west-2\",\"ap-shanghai\",\"ap-nanjing\",\"ap-guangzhou\"]) }}",'
                '"os":"{{ random.choice([\"Ubuntu\",\"Centos\",\"Debian\",\"TencentOS\"]) }}",'
                '"timestamp":{{ int(time.time() * 1000) }},'
                '"fields":{'
                '"redis_uptime_in_seconds":{{ random.randint(1,1000000) }},'
                '"redis_total_connections_received":{{ random.randint(1,1000000) }},'
                '"redis_expired_keys":{{ random.randint(1,1000000) }}'
                '}}'
            )
        else:
            # 当render为false时，使用f-string直接生成数据，避免模板渲染开销
            regions = ["us-west-2", "ap-shanghai", "ap-nanjing", "ap-guangzhou"]
            os_types = ["Ubuntu", "Centos", "Debian", "TencentOS"]
            data = (
                f'{{"host":"{socket.inet_ntoa(struct.pack("!I", random.randint(0xc0a80001, 0xc0a8ffff)))}",'
                f'"region":"{random.choice(regions)}",'
                f'"os":"{random.choice(os_types)}",'
                f'"timestamp":{int(time.time() * 1000)},'
                f'"fields":{{'
                f'"redis_uptime_in_seconds":{random.randint(1,1000000)},'
                f'"redis_total_connections_received":{random.randint(1,1000000)},'
                f'"redis_expired_keys":{random.randint(1,1000000)}'
                f'}}}}'
            )
    # 直接调用优化后的mock_data函数
    mock_data(repo, data, file, number, batch, gzip, workers, output_type, output_file, render)


import time
import random


class LogGenerator:
    """日志生成器类，用于生成不同类型的日志模板
    
    这个类抽象了不同日志类型的生成逻辑，支持多种日志类型，包括：
    - nginx: Nginx访问日志
    - java: Java应用日志
    - linux: Linux系统日志
    - apache: Apache访问日志
    - mysql: MySQL数据库日志
    - windows: Windows事件日志
    - mongodb: MongoDB数据库日志
    - docker: Docker容器日志
    
    每种日志类型都支持两种渲染模式：
    - render=True: 使用Jinja2模板引擎渲染，支持动态内容
    - render=False: 使用f-strings直接生成，性能更好
    
    使用示例：
    ```python
    generator = LogGenerator()
    nginx_log = generator.generate_log("nginx", render=True)
    java_log = generator.generate_log("java", render=False)
    ```
    """
    
    def __init__(self):
        """初始化日志生成器"""
        
        # 初始化日志类型映射
        self._log_generators = {
            "nginx": self._generate_nginx_log,
            "java": self._generate_java_log,
            "linux": self._generate_linux_log,
            "apache": self._generate_apache_log,
            "mysql": self._generate_mysql_log,
            "windows": self._generate_windows_log,
            "mongodb": self._generate_mongodb_log,
            "docker": self._generate_docker_log,
        }
    
    def get_supported_log_types(self):
        """获取支持的日志类型列表
        
        Returns:
            list: 支持的日志类型列表
        """
        return list(self._log_generators.keys())
    
    def generate_log(self, log_type="nginx", render=False):
        """生成指定类型的日志
        
        Args:
            log_type (str): 日志类型，支持 "nginx", "java", "linux", "apache", "mysql", "windows"
            render (bool): 是否使用Jinja2渲染，True使用模板渲染，False使用f-strings
            
        Returns:
            str: 生成的日志JSON字符串
        
        Raises:
            ValueError: 如果指定的日志类型不支持
        """
        if log_type not in self._log_generators:
            raise ValueError(f"不支持的日志类型: {log_type}，支持的类型: {', '.join(self.get_supported_log_types())}")
        
        # 调用对应的日志生成函数
        return self._log_generators[log_type](render)
    
    def _generate_nginx_log(self, render):
        """生成Nginx访问日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的Nginx日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ random.choice([\"192.168.1.1\", \"192.168.1.2\", \"192.168.1.3\", \"192.168.1.4\", \"192.168.1.5\"]) }} - - '
                '[{{ time.strftime(\"%d/%b/%Y:%H:%M:%S +0000\", time.localtime()) }}] '
                '\\\"{{ random.choice([\"GET\", \"POST\", \"PUT\"]) }} {{ random.choice([\"/\", \"/index.html\", \"/api/v1/users\", \"/login\", \"/static/css/main.css\"]) }} HTTP/1.1\\\" '
                '{{ random.choice([\"200\", \"201\", \"301\", \"302\", \"304\", \"400\", \"404\", \"500\"]) }} '
                '{{ random.randint(100, 10000) }} '
                '\\\"{{ random.choice([\"http://example.com\", \"http://referer.com\", \"-\"]) }}\\\" '
                '\\\"Mozilla/5.0 ({{ random.choice([\"Windows NT 10.0\", \"Macintosh\", \"Linux x86_64\", \"iPhone; CPU iPhone OS 14_0\"]) }}) '
                '{{ random.choice([\"Chrome/90.0.4430.212\", \"Safari/537.36\", \"Firefox/88.0\", \"Edge/91.0.864.48\"]) }}\\\"",' 
                '"host":"{{ random.choice([\"web-server-01\", \"web-server-02\", \"web-server-03\", \"web-server-04\", \"web-server-05\"]) }}",'
                '"origin": "nginx"'
                '}'
            )
        else:
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{random.choice(["192.168.1.1", "192.168.1.2", "192.168.1.3", "192.168.1.4", "192.168.1.5"])} - - '
                f'[{time.strftime("%d/%b/%Y:%H:%M:%S +0000", time.localtime())}] \\"{random.choice(["GET", "POST", "PUT", "DELETE"])} {random.choice(["/", "/index.html", "/api/v1/users", "/login", "/static/css/main.css"])} '
                f'HTTP/1.1\\" {random.choice(["200", "201", "301", "302", "304", "400", "404", "500"])} {random.randint(100, 10000)} '
                f'\\"-\\" \\"Mozilla/5.0 ({random.choice(["Windows NT 10.0", "Macintosh", "Linux x86_64", "iPhone; CPU iPhone OS 14_0"])}) '
                f'{random.choice(["Chrome/90.0.4430.212", "Safari/537.36", "Firefox/88.0", "Edge/91.0.864.48"])}\\"", '
                f'"host": "{random.choice(["web-server-01", "web-server-02", "web-server-03", "web-server-04", "web-server-05"])}", '
                f'"origin": "nginx"}}'
            )
        return data
    
    def _generate_java_log(self, render):
        """生成Java应用日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的Java日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ time.strftime(\"%Y-%m-%d %H:%M:%S,%f\"[:-3], time.localtime()) }} '
                '[{{ random.choice([\"main\", \"http-nio-8080-exec-1\", \"pool-1-thread-1\", \"AsyncTask-1\", \"RMI TCP Connection\"]) }}] '
                '{{ random.choice([\"INFO\", \"DEBUG\", \"WARN\", \"ERROR\", \"TRACE\"]) }} '
                '{{ random.choice([\"com.example.Controller\", \"org.springframework.web.servlet.DispatcherServlet\", \"com.example.service.UserService\", \"com.example.repository.UserRepository\", \"org.hibernate.SQL\"]) }} - '
                '{{ faker.sentence() }}",' 
                '"host":"{{ random.choice([\"app-server-01\", \"app-server-02\", \"app-server-03\", \"app-server-04\", \"app-server-05\"]) }}",'
                '"level":"{{ random.choice([\"INFO\", \"DEBUG\", \"WARN\", \"ERROR\", \"TRACE\"]) }}",'
                '"thread":"{{ random.choice([\"main\", \"http-nio-8080-exec-1\", \"pool-1-thread-1\", \"AsyncTask-1\", \"RMI TCP Connection\"]) }}",'
                '"class":"{{ random.choice([\"com.example.Controller\", \"org.springframework.web.servlet.DispatcherServlet\", \"com.example.service.UserService\", \"com.example.repository.UserRepository\", \"org.hibernate.SQL\"]) }}",'
                '"origin": "java"'
                '}'
            )
            
        else:
            log_levels = ["INFO", "DEBUG", "WARN", "ERROR", "TRACE"]
            threads = ["main", "http-nio-8080-exec-1", "pool-1-thread-1", "AsyncTask-1", "RMI TCP Connection"]
            classes = ["com.example.Controller", "org.springframework.web.servlet.DispatcherServlet", 
                      "com.example.service.UserService", "com.example.repository.UserRepository", "org.hibernate.SQL"]
            hosts = ["app-server-01", "app-server-02", "app-server-03", "app-server-04", "app-server-05"]
            
            level = random.choice(log_levels)
            thread = random.choice(threads)
            class_name = random.choice(classes)
            host = random.choice(hosts)
            message = f"Processing request ID {random.randint(10000, 99999)} for user {random.choice(['user1', 'admin', 'guest', 'customer'])}"
            
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{time.strftime("%Y-%m-%d %H:%M:%S,%f"[:-3], time.localtime())} '
                f'[{thread}] {level} {class_name} - {message}", '
                f'"host": "{host}", '
                f'"level": "{level}", '
                f'"thread": "{thread}", '
                f'"class": "{class_name}", '
                f'"origin": "java"}}'
            )
        return data
    
    def _generate_linux_log(self, render):
        """生成Linux系统日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的Linux日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ time.strftime(\"%b %d %H:%M:%S\", time.localtime()) }} '
                '{{ random.choice([\"localhost\", \"server1\", \"server2\", \"server3\", \"server4\"]) }} '
                '{{ random.choice([\"sshd\", \"systemd\", \"kernel\", \"cron\", \"nginx\", \"apache2\", \"mysql\"]) }}'
                '[{{ random.randint(1000, 9999) }}]: '
                '{{ faker.sentence() }}",' 
                '"host":"{{ random.choice([\"localhost\", \"server1\", \"server2\", \"server3\", \"server4\"]) }}",'
                '"program":"{{ random.choice([\"sshd\", \"systemd\", \"kernel\", \"cron\", \"nginx\", \"apache2\", \"mysql\"]) }}",'
                '"pid":{{ random.randint(1000, 9999) }},'
                '"origin": "linux"'
                '}'
            )
            
        else:
            hosts = ["localhost", "server1", "server2", "server3", "server4"]
            programs = ["sshd", "systemd", "kernel", "cron", "nginx", "apache2", "mysql"]
            
            host = random.choice(hosts)
            program = random.choice(programs)
            pid = random.randint(1000, 9999)
            messages = [
                f"User {random.choice(['root', 'admin', 'user1', 'guest'])} logged in from {random.choice(['192.168.1.1', '10.0.0.1', '172.16.0.1'])}",
                f"Started {random.choice(['Service', 'Process', 'Task'])} {random.randint(100, 999)}",
                f"Connection from {random.choice(['192.168.1.1', '10.0.0.1', '172.16.0.1'])} port {random.randint(1000, 65535)}",
                f"Failed password for {random.choice(['root', 'admin', 'user1', 'guest'])} from {random.choice(['192.168.1.1', '10.0.0.1', '172.16.0.1'])}",
                f"Out of memory: Kill process {random.randint(1000, 9999)}"
            ]
            message = random.choice(messages)
            
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{time.strftime("%b %d %H:%M:%S", time.localtime())} '
                f'{host} {program}[{pid}]: {message}", '
                f'"host": "{host}", '
                f'"program": "{program}", '
                f'"pid": {pid}, '
                f'"origin": "linux"}}'
            )
        return data
    
    def _generate_apache_log(self, render):
        """生成Apache访问日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的Apache日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ random.choice([\"192.168.1.1\", \"192.168.1.2\", \"192.168.1.3\", \"192.168.1.4\", \"192.168.1.5\"]) }} - {{ random.choice([\"user1\", \"admin\", \"-\"]) }} '
                '[{{ time.strftime(\"%d/%b/%Y:%H:%M:%S %z\", time.localtime()) }}] '
                '\\\"{{ random.choice([\"GET\", \"POST\", \"PUT\", \"DELETE\"]) }} {{ random.choice([\"/\", \"/index.php\", \"/wp-admin\", \"/images/logo.png\", \"/api/data\"]) }} HTTP/{{ random.choice([\"1.0\", \"1.1\", \"2.0\"]) }}\\\" '
                '{{ random.choice([\"200\", \"201\", \"301\", \"302\", \"304\", \"400\", \"403\", \"404\", \"500\", \"503\"]) }} '
                '{{ random.randint(100, 50000) }} '
                '\\\"{{ random.choice([\"http://example.com/page\", \"http://google.com/search?q=example\", \"https://www.bing.com/search?q=example\", \"-\"]) }}\\\" '
                '\\\"{{ random.choice([\"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36\", \"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1.1 Safari/605.1.15\", \"Mozilla/5.0 (X11; Linux x86_64; rv:89.0) Gecko/20100101 Firefox/89.0\"]) }}\\\"",' 
                '"host":"{{ random.choice([\"apache-server-01\", \"apache-server-02\", \"apache-server-03\", \"apache-server-04\", \"apache-server-05\"]) }}",'
                '"method":"{{ random.choice([\"GET\", \"POST\", \"PUT\", \"DELETE\"]) }}",'
                '"status":{{ random.choice([\"200\", \"201\", \"301\", \"302\", \"304\", \"400\", \"403\", \"404\", \"500\", \"503\"]) }},'
                '"bytes":{{ random.randint(100, 50000) }},'
                '"referer":"{{ random.choice([\"http://example.com/page\", \"http://google.com/search?q=example\", \"https://www.bing.com/search?q=example\", \"-\"]) }}",'
                '"origin": "apache"'
                '}'
            )
            
        else:
            ips = ["192.168.1.1", "192.168.1.2", "192.168.1.3", "192.168.1.4", "192.168.1.5"]
            users = ["user1", "admin", "-"]
            methods = ["GET", "POST", "PUT", "DELETE"]
            paths = ["/", "/index.php", "/wp-admin", "/images/logo.png", "/api/data"]
            http_versions = ["1.0", "1.1", "2.0"]
            statuses = ["200", "201", "301", "302", "304", "400", "403", "404", "500", "503"]
            referers = ["http://example.com/page", "http://google.com/search?q=example", "https://www.bing.com/search?q=example", "-"]
            user_agents = [
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
                "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1.1 Safari/605.1.15",
                "Mozilla/5.0 (X11; Linux x86_64; rv:89.0) Gecko/20100101 Firefox/89.0"
            ]
            hosts = ["apache-server-01", "apache-server-02", "apache-server-03", "apache-server-04", "apache-server-05"]
            
            ip = random.choice(ips)
            user = random.choice(users)
            method = random.choice(methods)
            path = random.choice(paths)
            http_version = random.choice(http_versions)
            status = random.choice(statuses)
            bytes_sent = random.randint(100, 50000)
            referer = random.choice(referers)
            user_agent = random.choice(user_agents)
            host = random.choice(hosts)
            
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{ip} - {user} '
                f'[{time.strftime("%d/%b/%Y:%H:%M:%S %z", time.localtime())}] \\"{method} {path} HTTP/{http_version}\\" '
                f'{status} {bytes_sent} \\"{referer}\\" \\"{user_agent}\\"", '
                f'"host": "{host}", '
                f'"method": "{method}", '
                f'"status": {status}, '
                f'"bytes": {bytes_sent}, '
                f'"referer": "{referer}", '
                f'"origin": "apache"}}'
            )
        return data
    
    def _generate_mongodb_log(self, render):
        """生成MongoDB数据库日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的MongoDB日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ time.strftime("%Y-%m-%dT%H:%M:%S.%fZ", time.localtime()) }} '
                '{{ random.choice(["I", "W", "E", "F", "D"]) }} '
                '{{ random.choice(["REPL", "NETWORK", "QUERY", "COMMAND", "STORAGE", "JOURNAL", "CONTROL"]) }} '
                '[{{ random.choice(["conn123456", "conn234567", "conn345678", "conn456789", "conn567890"]) }}] '
                '{{ random.choice(["Replica set member has gone into state PRIMARY", "Replica set member has gone into state SECONDARY", "Connection accepted", "Connection ended", "Slow query", "Index build started", "Index build completed", "Shutting down the WiredTiger storage engine", "Waiting for connections", "Received kill command"]) }}", '
                '"host":"{{ random.choice(["mongodb-server-01", "mongodb-server-02", "mongodb-server-03", "mongodb-server-04", "mongodb-server-05"]) }}", '
                '"severity":"{{ random.choice(["I", "W", "E", "F", "D"]) }}", '
                '"component":"{{ random.choice(["REPL", "NETWORK", "QUERY", "COMMAND", "STORAGE", "JOURNAL", "CONTROL"]) }}", '
                '"context":"{{ random.choice(["conn123456", "conn234567", "conn345678", "conn456789", "conn567890"]) }}",'
                '"origin": "mongodb"'
                '}'
            )
        else:
            # 日志级别
            severities = ["I", "W", "E", "F", "D"]
            
            # 组件
            components = ["REPL", "NETWORK", "QUERY", "COMMAND", "STORAGE", "JOURNAL", "CONTROL"]
            
            # 连接上下文
            contexts = ["conn123456", "conn234567", "conn345678", "conn456789", "conn567890"]
            
            # 消息
            messages = [
                "Replica set member has gone into state PRIMARY",
                "Replica set member has gone into state SECONDARY",
                "Connection accepted",
                "Connection ended",
                "Slow query",
                "Index build started",
                "Index build completed",
                "Shutting down the WiredTiger storage engine",
                "Waiting for connections",
                "Received kill command"
            ]
            
            # 主机
            hosts = ["mongodb-server-01", "mongodb-server-02", "mongodb-server-03", "mongodb-server-04", "mongodb-server-05"]
            
            # 随机选择值
            severity = random.choice(severities)
            component = random.choice(components)
            context = random.choice(contexts)
            message = random.choice(messages)
            host = random.choice(hosts)
            
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{time.strftime("%Y-%m-%dT%H:%M:%S.%fZ", time.localtime())} {severity} {component} [{context}] {message}", '
                f'"host": "{host}", '
                f'"severity": "{severity}", '
                f'"component": "{component}", '
                f'"context": "{context}", '  
                f'"origin": "mongodb"}}'
            )
        return data
    
    def _generate_mysql_log(self, render):
        """生成MySQL数据库日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的MySQL日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ time.strftime(\"%Y-%m-%d %H:%M:%S\", time.localtime()) }} '
                '{{ random.randint(1, 999999) }} '
                '[{{ random.choice([\"Note\", \"Warning\", \"Error\", \"System\"]) }}] '
                '{{ random.choice([\"[Server] Shutdown complete\", \"[InnoDB] Database was not shut down normally!\", \"[Server] /usr/sbin/mysqld: ready for connections.\", \"[Warning] IP address could not be resolved\", \"[Note] Event Scheduler: Loaded 0 events\", \"[System] [MY-010931] [Server] /usr/sbin/mysqld: ready for connections.\", \"[Warning] Aborted connection\", \"[Note] [MY-010311] [Server] Server initialized for online backups\", \"[ERROR] [MY-013183] [InnoDB] Assertion failure\", \"[Note] [MY-010051] [Server] Event Scheduler: scheduler thread started with id\"]) }}",' 
                '"host":"{{ random.choice([\"mysql-server-01\", \"mysql-server-02\", \"mysql-server-03\", \"mysql-server-04\", \"mysql-server-05\"]) }}",'
                '"thread_id":{{ random.randint(1, 999999) }},'
                '"level":"{{ random.choice([\"Note\", \"Warning\", \"Error\", \"System\"]) }}",'
                '"component":"{{ random.choice([\"Server\", \"InnoDB\", \"Connection\", \"System\", \"Warning\"]) }}",'
                '"origin": "mysql"'
                '}'
            )
            
        else:
            thread_ids = [random.randint(1, 999999) for _ in range(5)]
            levels = ["Note", "Warning", "Error", "System"]
            components = ["Server", "InnoDB", "Connection", "System", "Warning"]
            messages = [
                "[Server] Shutdown complete",
                "[InnoDB] Database was not shut down normally!",
                "[Server] /usr/sbin/mysqld: ready for connections.",
                "[Warning] IP address could not be resolved",
                "[Note] Event Scheduler: Loaded 0 events",
                "[System] [MY-010931] [Server] /usr/sbin/mysqld: ready for connections.",
                "[Warning] Aborted connection",
                "[Note] [MY-010311] [Server] Server initialized for online backups",
                "[ERROR] [MY-013183] [InnoDB] Assertion failure",
                "[Note] [MY-010051] [Server] Event Scheduler: scheduler thread started with id"
            ]
            hosts = ["mysql-server-01", "mysql-server-02", "mysql-server-03", "mysql-server-04", "mysql-server-05"]
            
            thread_id = random.choice(thread_ids)
            level = random.choice(levels)
            message = random.choice(messages)
            host = random.choice(hosts)
            component = random.choice(components)
            
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())} '
                f'{thread_id} [{level}] {message}", '
                f'"host": "{host}", '
                f'"thread_id": {thread_id}, '
                f'"level": "{level}", '
                f'"component": "{component}", '
                f'"origin": "mysql"}}'
            )
        return data
    
    def _generate_windows_log(self, render):
        """生成Windows事件日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的Windows事件日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ time.strftime(\"%Y-%m-%d %H:%M:%S\", time.localtime()) }} '
                '{{ random.choice([\"Information\", \"Warning\", \"Error\", \"Critical\", \"Verbose\"]) }} '
                '{{ random.choice([\"System\", \"Application\", \"Security\", \"Setup\", \"Windows PowerShell\"]) }} '
                '{{ random.choice([\"Service Control Manager\", \"Microsoft-Windows-Security-Auditing\", \"Microsoft-Windows-Kernel-General\", \"Microsoft-Windows-Winlogon\", \"Microsoft-Windows-GroupPolicy\"]) }} '
                '{{ random.randint(1000, 9999) }} '
                '{{ random.choice([\"The service entered the running state.\", \"Windows is shutting down.\", \"A user logged on to this computer.\", \"The Group Policy settings for the computer were processed successfully.\", \"The Windows Firewall service entered the running state.\", \"The Windows Update service failed to start.\", \"The system has resumed from sleep.\", \"A service was installed in the system.\", \"The system time has changed.\", \"The Windows Firewall has detected an unauthorized application attempting to access the network.\"]) }}",' 
                '"host":"{{ random.choice([\"win-server-01\", \"win-server-02\", \"win-server-03\", \"win-server-04\", \"win-server-05\"]) }}",'
                '"level":"{{ random.choice([\"Information\", \"Warning\", \"Error\", \"Critical\", \"Verbose\"]) }}",'
                '"log":"{{ random.choice([\"System\", \"Application\", \"Security\", \"Setup\", \"Windows PowerShell\"]) }}",'
                '"source":"{{ random.choice([\"Service Control Manager\", \"Microsoft-Windows-Security-Auditing\", \"Microsoft-Windows-Kernel-General\", \"Microsoft-Windows-Winlogon\", \"Microsoft-Windows-GroupPolicy\"]) }}",'
                '"event_id":{{ random.randint(1000, 9999) }},'
                '"origin": "windows"'
                '}'
            )
            
        else:
            levels = ["Information", "Warning", "Error", "Critical", "Verbose"]
            logs = ["System", "Application", "Security", "Setup", "Windows PowerShell"]
            sources = ["Service Control Manager", "Microsoft-Windows-Security-Auditing", "Microsoft-Windows-Kernel-General", 
                      "Microsoft-Windows-Winlogon", "Microsoft-Windows-GroupPolicy"]
            event_ids = [random.randint(1000, 9999) for _ in range(5)]
            messages = [
                "The service entered the running state.",
                "Windows is shutting down.",
                "A user logged on to this computer.",
                "The Group Policy settings for the computer were processed successfully.",
                "The Windows Firewall service entered the running state.",
                "The Windows Update service failed to start.",
                "The system has resumed from sleep.",
                "A service was installed in the system.",
                "The system time has changed.",
                "The Windows Firewall has detected an unauthorized application attempting to access the network."
            ]
            hosts = ["win-server-01", "win-server-02", "win-server-03", "win-server-04", "win-server-05"]
            
            level = random.choice(levels)
            log = random.choice(logs)
            source = random.choice(sources)
            event_id = random.choice(event_ids)
            message = random.choice(messages)
            host = random.choice(hosts)
            
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())} '
                f'{level} {log} {source} {event_id} {message}", '
                f'"host": "{host}", '
                f'"level": "{level}", '
                f'"log": "{log}", '
                f'"source": "{source}", '
                f'"event_id": {event_id}, '
                f'"origin": "windows"}}'
            )
        return data

    def _generate_docker_log(self, render):
        """生成Docker容器日志
        
        Args:
            render (bool): 是否使用Jinja2渲染
            
        Returns:
            str: 生成的Docker日志JSON字符串
        """
        if render:
            data = (
                '{"raw":"{{ time.strftime("%Y-%m-%dT%H:%M:%S.%fZ", time.localtime()) }} '
                '{{ random.choice(["stdout", "stderr"]) }} '
                '{{ random.choice(["F", "P"]) }} '
                '{{ random.choice(["2023-06-13 14:50:14.123 INFO", "2023-06-13 14:50:15.456 ERROR", "2023-06-13 14:50:16.789 WARN", "2023-06-13 14:50:17.012 DEBUG"]) }} '
                '{{ random.choice(["Starting container", "Container started", "Health check passed", "Connection established", "Processing request", "Request completed", "Memory usage: 75%", "CPU usage: 30%", "Disk I/O: 15MB/s", "Network I/O: 5MB/s", "Container stopping", "Container stopped"]) }}", '
                '"container_id":"{{ "%032x" % random.randrange(16**32) }}", '
                '"container_name":"{{ random.choice(["nginx", "redis", "mysql", "mongodb", "elasticsearch", "kibana", "prometheus", "grafana", "app", "api"]) }}-{{ random.randint(1, 5) }}", '
                '"image":"{{ random.choice(["nginx:latest", "redis:6.2", "mysql:8.0", "mongo:5.0", "elasticsearch:7.14", "kibana:7.14", "prom/prometheus:v2.30.0", "grafana/grafana:8.2.0", "myapp:1.0", "myapi:2.1"]) }}", '
                '"stream":"{{ random.choice(["stdout", "stderr"]) }}", '
                '"origin": "docker"'
                '}'
            )
        else:
            # 容器ID和名称
            container_ids = ["%032x" % random.randrange(16**32) for _ in range(5)]
            container_names = [f"{name}-{random.randint(1, 5)}" for name in ["nginx", "redis", "mysql", "mongodb", "elasticsearch", "kibana", "prometheus", "grafana", "app", "api"]]
            
            # 镜像名称
            images = ["nginx:latest", "redis:6.2", "mysql:8.0", "mongo:5.0", "elasticsearch:7.14", "kibana:7.14", "prom/prometheus:v2.30.0", "grafana/grafana:8.2.0", "myapp:1.0", "myapi:2.1"]
            
            # 日志级别和消息
            log_levels = ["INFO", "ERROR", "WARN", "DEBUG"]
            messages = [
                "Starting container", 
                "Container started", 
                "Health check passed", 
                "Connection established", 
                "Processing request", 
                "Request completed", 
                "Memory usage: 75%", 
                "CPU usage: 30%", 
                "Disk I/O: 15MB/s", 
                "Network I/O: 5MB/s", 
                "Container stopping", 
                "Container stopped"
            ]
            
            # 随机选择值
            container_id = random.choice(container_ids)
            container_name = random.choice(container_names)
            image = random.choice(images)
            stream = random.choice(["stdout", "stderr"])
            log_level = random.choice(log_levels)
            message = random.choice(messages)
            
            # 生成时间戳
            timestamp = time.strftime("%Y-%m-%dT%H:%M:%S.%fZ", time.localtime())
            
            # 使用双引号而不是单引号，确保生成有效的JSON
            data = (
                f'{{"raw": "{timestamp} {stream} {log_level[0]} {timestamp} {log_level} {message}", '
                f'"container_id": "{container_id}", '
                f'"container_name": "{container_name}", '
                f'"image": "{image}", '
                f'"stream": "{stream}", '
                f'"origin": "docker"}}'
            )
        return data



"""
AI大模型请求客户端

提供统一的AI大模型调用接口，支持多种模型和验证机制。
"""

import requests
import json
import time
from typing import Dict, Any, Optional, Callable, Union
from dataclasses import dataclass
from rich.console import Console
from .config import AIConfig, AIModelConfig
from .validators import ResponseValidator

# 创建console实例用于调试日志
console = Console()


@dataclass
class AIRequest:
    """AI请求数据结构"""
    messages: list
    model: Optional[str] = None
    max_tokens: Optional[int] = None
    temperature: Optional[float] = None
    stream: bool = False
    extra_params: Optional[Dict[str, Any]] = None


@dataclass
class AIResponse:
    """AI响应数据结构"""
    content: str
    raw_response: Dict[str, Any]
    model: str
    usage: Optional[Dict[str, Any]] = None
    finish_reason: Optional[str] = None
    request_id: Optional[str] = None


class AIClient:
    """AI大模型请求客户端"""
    
    def __init__(self, config_path: str = None, model_name: str = None, system_prompt: str = None):
        """
        初始化AI客户端
        
        Args:
            config_path: 配置文件路径
            model_name: 模型名称，如果为None则使用默认模型
            system_prompt: 系统提示词，将在每次对话开始时自动添加
        """
        self.config_manager = AIConfig(config_path)
        self.model_name = model_name or self.config_manager.get_default_model()
        self.model_config = self.config_manager.get_model_config(self.model_name)
        self.system_prompt = system_prompt
        self.session = requests.Session()
        
        # 设置默认请求头
        if self.model_config.headers:
            self.session.headers.update(self.model_config.headers)
        
        # 设置认证
        self._setup_auth()
    
    def _setup_auth(self):
        """设置认证信息"""
        if self.model_config.api_key:
            if 'openai' in self.model_name.lower():
                self.session.headers['Authorization'] = f'Bearer {self.model_config.api_key}'
            elif 'claude' in self.model_name.lower() or 'anthropic' in self.model_name.lower():
                self.session.headers['x-api-key'] = self.model_config.api_key
            else:
                # 通用认证方式
                self.session.headers['Authorization'] = f'Bearer {self.model_config.api_key}'
    
    def _prepare_request_data(self, request: AIRequest) -> Dict[str, Any]:
        """
        准备请求数据
        
        Args:
            request: AI请求对象
            
        Returns:
            Dict: 请求数据
        """
        data = {
            'model': request.model or self.model_config.model,
            'max_tokens': request.max_tokens or self.model_config.max_tokens,
            'temperature': request.temperature or self.model_config.temperature,
        }
        
        # 根据不同的模型类型处理消息格式
        if 'openai' in self.model_name.lower():
            data['messages'] = request.messages
            if request.stream:
                data['stream'] = True
        elif 'claude' in self.model_name.lower() or 'anthropic' in self.model_name.lower():
            # Claude API格式
            if request.messages:
                # 转换消息格式
                if len(request.messages) == 1 and request.messages[0].get('role') == 'user':
                    data['messages'] = request.messages
                else:
                    data['messages'] = request.messages
        elif 'qwen' in self.model_name.lower():
            # Qwen API格式
            data['messages'] = request.messages
            if request.stream:
                data['stream'] = True
        else:
            # 通用格式
            data['messages'] = request.messages
            if request.stream:
                data['stream'] = True
        
        # 添加额外参数
        if request.extra_params:
            data.update(request.extra_params)
        
        if self.model_config.extra_params:
            data.update(self.model_config.extra_params)        
        return data
    
    def _parse_response(self, response: requests.Response) -> AIResponse:
        """
        解析响应数据
        
        Args:
            response: HTTP响应对象
            
        Returns:
            AIResponse: 解析后的AI响应对象
        """
        try:
            response_data = response.json()
        except json.JSONDecodeError:
            raise Exception(f"无法解析响应JSON: {response.text}")
        
        # 根据不同模型解析响应
        if 'openai' in self.model_name.lower():
            return self._parse_openai_response(response_data)
        elif 'claude' in self.model_name.lower() or 'anthropic' in self.model_name.lower():
            return self._parse_claude_response(response_data)
        else:
            return self._parse_generic_response(response_data)
    
    def _parse_openai_response(self, data: Dict[str, Any]) -> AIResponse:
        """解析OpenAI响应"""
        if 'choices' not in data or not data['choices']:
            raise Exception("OpenAI响应中没有choices字段")
        
        choice = data['choices'][0]
        content = choice.get('message', {}).get('content', '')
        finish_reason = choice.get('finish_reason')
        
        return AIResponse(
            content=content,
            raw_response=data,
            model=data.get('model', self.model_config.model),
            usage=data.get('usage'),
            finish_reason=finish_reason,
            request_id=data.get('id')
        )
    
    def _parse_claude_response(self, data: Dict[str, Any]) -> AIResponse:
        """解析Claude响应"""
        if 'content' not in data:
            raise Exception("Claude响应中没有content字段")
        
        content_list = data['content']
        if isinstance(content_list, list) and content_list:
            content = content_list[0].get('text', '')
        else:
            content = str(content_list)
        
        return AIResponse(
            content=content,
            raw_response=data,
            model=data.get('model', self.model_config.model),
            usage=data.get('usage'),
            finish_reason=data.get('stop_reason'),
            request_id=data.get('id')
        )
    
    def _parse_generic_response(self, data: Dict[str, Any]) -> AIResponse:
        """解析通用响应"""
        content = ''
        finish_reason = None
        
        # 首先尝试OpenAI格式 (choices[0].message.content)
        if 'choices' in data and data['choices'] and len(data['choices']) > 0:
            choice = data['choices'][0]
            if 'message' in choice and 'content' in choice['message']:
                content = choice['message']['content']
                finish_reason = choice.get('finish_reason')
        
        # 如果没有找到，尝试其他可能的内容字段
        if not content:
            for field in ['content', 'text', 'response', 'output']:
                if field in data:
                    content = str(data[field])
                    break
        
        return AIResponse(
            content=content,
            raw_response=data,
            model=data.get('model', self.model_config.model),
            usage=data.get('usage'),
            finish_reason=finish_reason or data.get('finish_reason'),
            request_id=data.get('id')
        )
    
    def chat(self, 
             messages: Union[str, list], 
             model: str = None,
             max_tokens: int = None,
             temperature: float = None,
             validator: ResponseValidator = None,
             callback: Callable[[AIResponse], Any] = None,
             **kwargs) -> AIResponse:
        """
        发送聊天请求
        
        Args:
            messages: 消息内容，可以是字符串或消息列表
            model: 模型名称
            max_tokens: 最大token数
            temperature: 温度参数
            validator: 响应验证器
            callback: 响应回调函数
            **kwargs: 其他参数
            
        Returns:
            AIResponse: AI响应对象
        """

        # 处理消息格式
        if isinstance(messages, str):
            messages = [{"role": "user", "content": messages}]
        
        # 如果设置了系统提示词，且消息列表中没有系统消息，则添加系统提示词
        if self.system_prompt:
            if not messages or messages[0].get("role") != "system":
                messages.insert(0, {"role": "system", "content": self.system_prompt})
        
        
        # 创建请求对象
        request = AIRequest(
            messages=messages,
            model=model,
            max_tokens=max_tokens,
            temperature=temperature,
            extra_params=kwargs
        )
        
        return self.send_request(request, validator, callback)
    
    def send_request(self, 
                    request: AIRequest, 
                    validator: ResponseValidator = None,
                    callback: Callable[[AIResponse], Any] = None) -> AIResponse:
        """
        发送AI请求
        
        Args:
            request: AI请求对象
            validator: 响应验证器
            callback: 响应回调函数
            
        Returns:
            AIResponse: AI响应对象
        """
        
        try:
            # 准备请求数据
            request_data = self._prepare_request_data(request)
            
            response = self.session.post(
                self.model_config.endpoint,
                json=request_data,
                timeout=self.model_config.timeout
            )

            # 检查HTTP状态
            if not response.ok:
                console.print(f"[bold red][DEBUG][/bold red] HTTP请求失败: {response.status_code}")
                console.print(f"[bold red][DEBUG][/bold red] 错误响应内容: {response.text[:500]}...")
                raise Exception(f"HTTP请求失败: {response.status_code} - {response.text}")
            
            
            # 解析响应
            ai_response = self._parse_response(response)
            
            # 验证响应
            if validator:
                is_valid, errors = validator.validate(ai_response.raw_response)
                if not is_valid:
                    raise Exception(f"响应验证失败: {'; '.join(errors)}")
            
            # 执行回调
            if callback:
                callback(ai_response)
            
            return ai_response
            
        except requests.exceptions.Timeout:
            raise Exception(f"请求超时 ({self.model_config.timeout}秒)")
        except requests.exceptions.ConnectionError:
            raise Exception("连接错误，请检查网络和端点地址")
        except Exception as e:
            console.print(f"[bold red][DEBUG][/bold red] 请求异常: {type(e).__name__}: {e}")
            raise Exception(f"AI请求失败: {e}")
    
    def stream_chat(self, 
                   messages: Union[str, list],
                   model: str = None,
                   max_tokens: int = None,
                   temperature: float = None,
                   callback: Callable[[str], None] = None,
                   **kwargs):
        """
        流式聊天请求
        
        Args:
            messages: 消息内容
            model: 模型名称
            max_tokens: 最大token数
            temperature: 温度参数
            callback: 流式数据回调函数
            **kwargs: 其他参数
            
        Yields:
            str: 流式响应内容片段
        """
        
        # 处理消息格式
        if isinstance(messages, str):
            messages = [{"role": "user", "content": messages}]
        
        
        # 如果设置了系统提示词，且消息列表中没有系统消息，则添加系统提示词
        if self.system_prompt and (not messages or messages[0].get("role") != "system"):
            messages.insert(0, {"role": "system", "content": self.system_prompt})

        # 创建请求对象
        request = AIRequest(
            messages=messages,
            model=model,
            max_tokens=max_tokens,
            temperature=temperature,
            stream=True,
            extra_params=kwargs
        )
        
        try:
            
            # 准备请求数据
            request_data = self._prepare_request_data(request)
            
            # 发送流式请求
            start_time = time.time()
            
            response = self.session.post(
                self.model_config.endpoint,
                json=request_data,
                timeout=self.model_config.timeout,
                stream=True
            )
            
            
            # 检查HTTP状态
            if not response.ok:
                console.print(f"[bold red][DEBUG][/bold red] 错误响应内容: {response.text[:500]}...")
                raise Exception(f"HTTP请求失败: {response.status_code} - {response.text}")
            
            chunk_count = 0
            
            # 处理流式响应
            for line in response.iter_lines():
                if line:
                    line_text = line.decode('utf-8')
                    
                    if line_text.startswith('data: '):
                        data_text = line_text[6:]
                        
                        if data_text.strip() == '[DONE]':
                            break
                        
                        try:
                            data = json.loads(data_text)
                            content = self._extract_stream_content(data)
                            if content:
                                chunk_count += 1
                                if callback:
                                    callback(content)
                                yield content
                        except json.JSONDecodeError:
                            console.print(f"[bold yellow][DEBUG][/bold yellow] 跳过无效的JSON数据: {data_text[:100]}...")
                            continue
            
        except requests.exceptions.Timeout:
            console.print(f"[bold red][DEBUG][/bold red] 流式请求超时异常")
            raise Exception(f"请求超时 ({self.model_config.timeout}秒)")
        except requests.exceptions.ConnectionError:
            console.print(f"[bold red][DEBUG][/bold red] 流式请求连接错误异常")
            raise Exception("连接错误，请检查网络和端点地址")
        except Exception as e:
            console.print(f"[bold red][DEBUG][/bold red] 流式请求异常: {type(e).__name__}: {e}")
            raise Exception(f"流式请求失败: {e}")
    
    def _extract_stream_content(self, data: Dict[str, Any]) -> str:
        """从流式数据中提取内容"""
        if 'openai' in self.model_name.lower():
            choices = data.get('choices', [])
            if choices:
                delta = choices[0].get('delta', {})
                return delta.get('content', '')
        elif 'claude' in self.model_name.lower():
            return data.get('delta', {}).get('text', '')
        elif 'qwen' in self.model_name.lower():
            # 处理qwen模型的流式响应格式
            # 尝试多种可能的格式
            
            # 格式1: choices[0].delta.content (类似OpenAI)
            choices = data.get('choices', [])
            if choices:
                delta = choices[0].get('delta', {})
                content = delta.get('content', '')
                if content:
                    return content
            
            # 格式2: 直接的content字段
            content = data.get('content', '')
            if content:
                return content
            
            # 格式3: text字段
            content = data.get('text', '')
            if content:
                return content
            
            # 格式4: output字段
            content = data.get('output', '')
            if content:
                return content
        
        return ''
    
    def switch_model(self, model_name: str):
        """
        切换模型
        
        Args:
            model_name: 新的模型名称
        """
        self.model_name = model_name
        self.model_config = self.config_manager.get_model_config(model_name)
        
        # 重新设置认证
        self.session.headers.clear()
        if self.model_config.headers:
            self.session.headers.update(self.model_config.headers)
        self._setup_auth()
    
    def get_available_models(self) -> list:
        """获取可用的模型列表"""
        return self.config_manager.list_models()
    
    def get_current_model(self) -> str:
        """获取当前使用的模型"""
        return self.model_name


import argcomplete
import importlib.metadata
from mando import command, main, arg
import time
from datetime import datetime
import sys
from ketacli.sdk.base.client import *
from ketacli.sdk.base.search import search_spl, search_spl_meta, search_pql
from ketacli.sdk.request.list import list_assets_request, list_admin_request
from ketacli.sdk.request.create import create_asset_request
from ketacli.sdk.request.get import get_asset_by_id_request
from ketacli.sdk.request.update import update_asset_request
from ketacli.sdk.request.delete import delete_asset_request
from ketacli.sdk.request.export import export_asset_request
from ketacli.sdk.request.asset_map import get_resources
from ketacli.sdk.base.config import *
from ketacli.sdk.output.output import list_output, describe_output, get_asset_output
from ketacli.sdk.output.output import search_result_output
from ketacli.sdk.output.format import format_table
from ketacli.sdk.output.output import rs_output_all, rs_output_one
from ketacli.sdk.util import parse_url_params, Template
from rich.console import Console
from rich.live import Live
from rich.markdown import Markdown
from ketacli.sdk.chart.layout import LayoutChart
import importlib.resources as pkg_resources
from concurrent.futures import ProcessPoolExecutor
from multiprocessing import Manager
from rich.progress import Progress
import copy

# 导入mock模块中的函数
from ketacli.mock.mock_data import mock_data, mock_log, mock_metrics, generate_and_upload

# 导入测试模块
from ketacli.sdk.test.cli import test_command
from ketacli.sdk.test.test_runner import run_test_with_assertion

# 导入AI模块
from ketacli.sdk.ai import AIClient, AIConfig, ResponseValidator

console = Console()


@command
def login(name="keta", endpoint="http://localhost:9000", token=""):
    """Login to ketadb, cache authentication info to ~/.keta/config.yaml

    :param repository: Repository to push to.
    :param -n, --name: The login account name. Defaults to "keta".
    :param -e, --endpoint: The ketadb endpoint. Defaults to "http://localhost:9000".
    :param -t, --token: Your keta api token, create from ketadb webui. Defaults to "".
    """
    do_login(name=name, endpoint=endpoint, token=token)


@command
@arg("name", type=str,
     completer=lambda prefix, **kwd: [x['name'] for x in list_clusters() if x['name'].startswith(prefix)])
def logout(name=None):
    """Logout from ketadb, clear authentication info

    :param -n, --name: logout from which account
    """
    do_logout(name)


@command('list')
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def _list(asset_type, groupId=-1, order="desc", pageNo=1, pageSize=10, prefix="", sort="updateTime", fields="",
          format=None, raw=False, lang=None, extra=None, watch=False, interval=3.0, show_all_fields=False, test=False):
    """List asset (such as repo,sourcetype,metric...) from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -l, --pageSize: Limit the page size.
    :param --pageNo: Limit the page number.
    :param --prefix: Fuzzy query filter.
    :param --sort: The field used to order by
    :param --order: The sort order, desc|asc
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --groupId: The resource group id.
    :param --raw: Prettify the time field or output the raw timestamp, if specified, output the raw format
    :param --lang: Choose the language preference of return value
    :param -e, --extra: extra query filter, example: include_defaults=true,flat_settings=true
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    :param -a, --show_all_fields: all fields to display
    :param --test: Enable test mode for assertion control
    """
    extra_dict = {}
    if extra is not None:
        # 解析 url 参数为 dict
        extra_dict = parse_url_params(extra)

    # console.print(f"list {asset_type} with params: {extra_dict}")

    def generate_table():
        req = list_assets_request(
            asset_type, groupId, order, pageNo, pageSize, prefix, sort, lang, **extra_dict)
        resp = request_get(req["path"], req["query_params"],
                           req["custom_headers"]).json()
        output_fields = req.get("default_fields", [])
        field_aliases = req.get("field_aliases", {})
        field_converters = req.get("field_converters", {})
        if show_all_fields:
            output_fields = []
        if len(fields.strip()) > 0:
            output_fields = fields.strip().split(",")
        table = list_output(asset_type, output_fields=output_fields, resp=resp, field_aliases=field_aliases, field_converters=field_converters)
        if not table:
            return None
        return format_table(table, format, not raw)

    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "list"
        args.params = {"groupId": groupId, "order": order, "pageNo": pageNo, "pageSize": pageSize, 
                      "prefix": prefix, "sort": sort, "fields": fields, "lang": lang}
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()
    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any {asset_type}")
        else:
            console.print(table, overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
def admin(asset_type, format="json", extra=None, watch=False, interval=3.0):
    """List asset (such as repo,sourcetype,metric...) from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -f, --format: The output format, text|json|csv|html|latex
    :param -e, --extra: extra query filter, example: include_defaults=true,flat_settings=true
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    """
    extra_dict = {}
    if extra is not None:
        # 解析 url 参数为 dict
        extra_dict = parse_url_params(extra)

    def generate_table():
        req = list_admin_request(asset_type, **extra_dict)
        resp = request_get(req["path"], req["query_params"], req["custom_headers"]).json()
        output_fields = []
        table = list_output(asset_type, output_fields=output_fields, resp=resp)
        return format_table(table, format)

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()
    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any {asset_type}")
        else:
            console.print(table)


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def get(asset_type, asset_id, fields="", format=None, lang=None, extra=None):
    """Get asset detail info from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param asset_id: The unique id of asset. (such as id or name...)
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --lang: Choose the language preference of return value
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    extra_args_map['name'] = asset_id

    req = get_asset_by_id_request(
        asset_type=asset_type, asset_id=asset_id, lang=lang, **extra_args_map)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    if format == "json":
        console.print(json.dumps(resp, indent=2, ensure_ascii=False))
        return

    output_fields = []
    if len(fields.strip()) > 0:
        output_fields = fields.strip().split(",")
    table = get_asset_output(output_fields=output_fields, resp=resp)
    table.align = "l"
    if table is None:
        console.print(f"we cannot find any {asset_type}")
    else:
        console.print(format_table(table, format), overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def describe(asset_type, format=None):
    """Describe the schema of asset type

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -f, --format: The output format, text|json|csv|html|latex
    """

    req = list_assets_request(asset_type)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    table = describe_output(asset_type, resp=resp)
    if table is None:
        console.print(f"we cannot find any {asset_type}")
    else:
        console.print(format_table(table, format), overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
def search(spl, start=None, end=None, limit=100, format=None, raw=False, watch=False, interval=3.0):
    """Search spl from ketadb

    :param spl: The spl query
    :param --start: The start time. Time format "2024-01-02 10:10:10"
    :param --end: The start time. Time format "2024-01-02 10:10:10"
    :param -l, --limit: The limit size of query result
    :param -f, --format: The output format, table|text|json|csv|html|latex
    :param --raw: Prettify the time field or output the raw timestamp, if specified, output the raw format
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    """
    if start is not None:
        start = datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
    if end is not None:
        end = datetime.strptime(end, '%Y-%m-%d %H:%M:%S')

    def generate_table():
        resp = search_spl(spl=spl, start=start, end=end, limit=limit)
        return format_table(search_result_output(resp), format, not raw)

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()

    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any data")
        else:
            console.print(table, overflow="fold")


@command
def benchmark(type="spl", query=None, start: float = 0.0, end: float = None, limit=None, cnt=1, workers=1, base_url="",
              window=0):
    """benchmark spl from ketadb
    :param spl: The spl query
    :param --start: The start time
    :param --end: The start time
    :param -l, --limit: The limit size of query result
    :param -c, --cnt: The count of benchmark
    :param -w, --workers: The count of workers
    :param -b, --base_url: The base url of pql
    :param --window: The window of benchmark
    """

    result = {
        "type": type,
        "query": query,
        "cnt": cnt,
        "results": [],
        "waiting": 0,
        'request_cnt': cnt
    }
    futures = []
    started = time.time()
    if not end:
        end = time.time()
    if isinstance(query, str):
        querys = [query]
    elif isinstance(query, list):
        querys = query
    else:
        raise Exception("query must be str or list")
    with Progress() as progress:
        if window:
            new_cnt = int((end - start) / window) * cnt * len(querys)
            result['request_cnt'] = new_cnt
        else:
            new_cnt = cnt * len(querys)
            result['request_cnt'] = new_cnt
        task = progress.add_task(f"[green]Benchmarking total: {new_cnt}...", total=new_cnt)
        with ProcessPoolExecutor(max_workers=workers) as executor:

            if not window:
                for _ in range(cnt):
                    for query in querys:
                        if type == "spl":
                            futures.append(
                                executor.submit(search_spl_meta, spl=query, start=start, end=end, limit=limit))
                        elif type == "pql":
                            futures.append(
                                executor.submit(search_pql, base_url=base_url, pql=query, start=start, end=end,
                                                limit=limit))
            else:

                while True:
                    new_end = start + window
                    print(datetime.fromtimestamp(start).strftime('%d-%H:%M:%S'),
                          datetime.fromtimestamp(new_end).strftime('%H:%M:%S'), )
                    if new_end > end:
                        break
                    for query in querys:
                        if type == "spl":
                            for _ in range(cnt):
                                futures.append(
                                    executor.submit(search_spl_meta, spl=query, start=start, end=new_end, limit=limit))
                        elif type == "pql":
                            for _ in range(cnt):
                                futures.append(
                                    executor.submit(search_pql, base_url=base_url, pql=query, start=start, end=new_end,
                                                    limit=limit))
                    start += window

            for future in futures:
                resp = future.result()
                result["results"].append(resp)
                result["avg_duration"] = sum([x["duration"] for x in result["results"]]) / len(result["results"])
                progress.update(task, advance=1)
    result["waiting"] = time.time() - started
    result["total_duration"] = round(sum([x["duration"] for x in result["results"] if "duration" in x]))
    result['totalSize'] = sum([x["resultSize"] for x in result["results"] if "resultSize" in x])
    console.print(result)


@command
def benchmark_for_file(type='spl', file_path=None, start=0, end=0, limit=None, cnt=1, workers=1, base_url="",
                       window=0):
    """从文件批量执行性能测试
    通过读取包含多个查询语句的文件，进行批量压力测试

    :param type: 测试类型 spl/pql
    :param file_path: 包含查询语句的文件路径，每行一个查询
    :param start: 测试时间范围起始时间戳
    :param end: 测试时间范围结束时间戳
    :param limit: 单次查询结果限制数
    :param cnt: 每个查询的重复测试次数
    :param workers: 并发工作进程数
    :param base_url: PQL服务基础地址（当type=pql时生效）
    :param window: 时间窗口大小（秒），0表示不分割时间范围

    示例：
    ketacli benchmark_for_file --file queries.txt --type spl --workers 4 --cnt 10
    """
    with open(file_path, "r") as f:
        querys = f.readlines()

    benchmark(type=type, query=querys, start=start, end=end, limit=limit, cnt=cnt, workers=workers, base_url=base_url,
              window=window)


@command
def plot(spl, start=None, end=None, limit=100, interval=3.0,
         type="line", title=None, x_label="Time", y_label="Value", x_field="_time",
         y_field="value", group_field="", extra=None, theme=None, ):
    """plot chart of a single plot.

    :param spl: The spl query
    :param --start: The start time. Time format "2024-01-02 10:10:10"
    :param --end: The start time. Time format "2024-01-02 10:10:10"
    :param -l, --limit: The limit size of query result
    :param --interval: refresh the resource change
    :param -t, --type: plot type, line|bar|scatter
    :param --title: plot title
    :param --x_label: x label
    :param --y_label: y label
    :param -x, --x_field: x field
    :param -y, --y_field: y field
    :param -g, --group_field: group field
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if start is not None:
        start = datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
    if end is not None:
        end = datetime.strptime(end, '%Y-%m-%d %H:%M:%S')
    if not title:
        title = spl
    extra_args_map = {}
    if extra:
        extra_args_map = parse_url_params(extra)

    config = {
        "layout": {
            "rows": 1,
            "columns": 1
        },
        "charts": [
            {
                "type": type,
                "title": title,
                "x_label": x_label,
                "y_label": y_label,
                "x_field": x_field,
                "y_field": y_field,
                "group_field": group_field,
                "plot_type": type,
                "extra_args": extra_args_map,
                "position": "row0-col0",
                "spl": spl,
                "start": start,
                "end": end,
                "limit": limit,
                "theme": theme,
                "interval": interval
            }
        ]
    }
    chart = LayoutChart(config)
    chart.live(interval=interval, theme=theme)


@command
@arg("chart", type=str, completer=lambda prefix, **kwd: [
    x.replace('.yaml', '').replace('.yml', '')
    for x in os.listdir(str(pkg_resources.files('ketacli').joinpath('charts'))) if x.startswith(prefix)
])
@arg("theme", type=str,
     completer=lambda prefix, **kwd: [
         x for x in ["default", "dark", "clear", "pro", "matrix", "windows",
                     "retro", "elegant", "mature", "dreamland", "grandpa",
                     "salad", "girly", "serious", "sahara", 'scream'] if x.startswith(prefix)
     ])
def dashboard(chart=None, file_path=None, interval=30, theme="", disable_auto_refresh=True):
    """plot dashboard from yaml file

    :param -c, --chart: The chart name, such as monitor
    :param -f, --file_path: The file path
    :param --interval: refresh the chart change, default 30s
    :param --theme: setting the theme, such as default|dark|clear|pro|matrix|windows|retro|elegant|mature|dreamland|grandpa|salad|girly|serious|sahara|scream
    :param -d, --disable_auto_refresh: auto refresh the chart change
    """
    if file_path is None and chart is None:
        console.print(f"Please specify file path with --file or --chart")
        return
    if chart is not None:
        file_path = str(pkg_resources.files('ketacli').joinpath('charts', f"{chart}.yaml"))
    config = yaml.safe_load(open(file_path, encoding="utf-8"))
    chart = LayoutChart(config)
    chart.live(interval=interval, theme=theme, disable_auto_refresh=disable_auto_refresh)


@command
def insert(repo="default", data=None, file=None):
    """Upload data to specified repo

    :param --repo: The target repo
    :param --data: The json string data [{"raw":"this is text", "host": "host-1"}]
    :param --file: Upload json text from file path.
    """
    if repo is None:
        console.print(f"Please specify target repo with --repo")
        return
    if data is None and file is None:
        console.print(f"Please use --data or --file to specify data to upload")
        return

    if file is not None:
        f = open(file, encoding="utf-8")
        data = f.read()

    query_params = {
        "repo": repo,
    }
    resp = request_post("data", json.loads(data), query_params).json()
    console.print(resp, overflow="fold")


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def create(asset_type, name=None, data=None, file=None, extra=None, test=False):
    """Create asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name
    :param --data: The json string data {...}
    :param --file: Upload json text from file path.
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --test: Enable test mode for assertion control
    """
    if data is None and file is None:
        content = {}
    else:
        content = data
        if file is not None:
            f = open(file, encoding="utf-8")
            content = f.read()
        try:
            content = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    for k, v in extra_args_map.items():
        if str(v).startswith('@'):
            c = open(v[1:], encoding="utf-8").read().replace("\n", "\\n").replace("\"", "\\\"")
            extra_args_map[k] = c
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')
    
    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "create"
        args.params = {"name": name, "data": content}
        if extra_args_map:
            args.params.update(extra_args_map)
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return

    try:
        req = create_asset_request(asset_type, name, content, **extra_args_map)
        resp = request(req["method"], req["path"], data=req['data']).json()
        console.print(json.dumps(resp, ensure_ascii=False), overflow="fold")
    except Exception as e:
        console.print(f"create asset {name} failed, error: {e}")


def get_operation_type(prefix, **kwargs):
    operators = []
    for x in get_resources():
        if x == kwargs.get('parsed_args').asset_type:
            methods = get_resources()[x].get('methods')
            operators = methods.keys()
    operators = [x for x in operators if
                 x not in ['list', 'create', 'update', 'delete', 'download', 'get'] and x.startswith(prefix)]
    return operators


@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
@command
@arg("operation", type=str, completer=get_operation_type)
def update(asset_type, name=None, operation="update", data=None, file=None, extra=None, test=False):
    """Update asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name
    :param -d, --data: The json string data {...}
    :param -f, --file: Upload json text from file path.
    :param -o, --operation: operation type, such as open, close, update, delete
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --test: Enable test mode for assertion control
    """
    if data is None and file is None:
        data = {}
    else:
        content = data
        if file is not None:
            f = open(file, encoding="utf-8")
            content = f.read()

        try:
            data = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')
    
    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "update"
        args.params = {"name": name, "operation": operation, "data": data}
        if extra_args_map:
            args.params.update(extra_args_map)
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return
    
    req = update_asset_request(asset_type, operation, name, data, **extra_args_map)
    resp = request(req["method"], req["path"], data=req['data']).json()
    console.print(resp, overflow="fold")


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def download(asset_type, extra=None, base_path="./"):
    """export file asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --base_path: the file save path
    """
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}

    req = export_asset_request(asset_type, **extra_args_map)
    download_file(req["path"], save_path=base_path)


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def delete(asset_type, name=None, data=None, file=None, extra=None, test=False):
    """Delete asset

    :param --asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name or id
    :param -d, --data: The json string data {...}
    :param -f, --file: Upload json text from file path.
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --test: Enable test mode for assertion control
    """
    if data is None and file is None:
        data = {}
    else:
        content = data
        if file is not None:
            f = open(file, encoding="utf8")
            content = f.read()

        try:
            data = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return

    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')

    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "delete"
        args.params = {"name": name, "data": data}
        if extra_args_map:
            args.params.update(extra_args_map)
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return

    req = delete_asset_request(asset_type, name, data, **extra_args_map)
    resp = request(req["method"], req["path"], data=req['data']).json()
    console.print(resp, overflow="fold")


@command
@arg('type', type=str, completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def rs(type=None, format="table"):
    """Show resource info

    :param -t, --type: The target asset type, such as repo, sourcetype ...
    :param -f, --format: The output format, text, json ...
    """
    resources = get_resources()
    if type is None:
        table = rs_output_all(resources)
    else:
        table = rs_output_one(type, resources.get(type))
    console.print(format_table(table, format=format), overflow="fold")


@command
@arg('name', '-n', '--name', type=str,
     completer=lambda prefix, **kwd: [x['name'] for x in list_clusters() if x['name'].startswith(prefix)])
@arg('operation', type=str,
     completer=lambda prefix, **kwd: [x for x in ['list-clusters', 'set-default', 'delete-cluster'] if
                                      x.startswith(prefix)])
def config(operation, name=None):
    """Show keta cluster info

    :param operation: The target operation, such as list-clusters, set-default, delete-cluster
    :param -n, --name: setting or delete default cluster
    """
    resp = []
    if operation not in ["list-clusters", "set-default", "delete-cluster"]:
        console.print(f"Please specify operation, such as list-clusters, set-default, delete-cluster", style="red")
        return
    if operation == "list-clusters":
        resp = list_clusters()
    elif operation == "set-default":
        resp = set_default_cluster(name)
    elif operation == "delete-cluster":
        resp = delete_cluster(name)
    if not resp:
        console.print("No response")
        exit()
    table = list_output('cluster', [], resp=resp)
    console.print(format_table(table, "table"), overflow="fold")


@command
def version():
    _version = importlib.metadata.version('ketacli')
    console.print(_version, overflow="fold")


@command
@arg('overflow', '-o', '--overflow', type=str,
     completer=lambda prefix, **kwd: [x for x in ["fold", "crop", "ellipsis", "ignore"] if x.startswith(prefix)])
def isearch(page_size=10, limit=500, overflow="ellipsis"):
    """
    interactive search, 该模式仅支持在界面化系统中允许，无法通过 ssh 连接到服务器来运行

    :params --page_size: page size, default 10
    :params --limit: search data number, default 500
    :params --overflow: 文本超出列宽度时的处理方式, default ellipsis ，可选 ["fold", "crop", "ellipsis", "ignore"]

    """
    try:
        from ketacli.sdk.chart.interactive_search import InteractiveSearch
        InteractiveSearch().run()
    except Exception as e:
        console.print_exception()

@command
def metric(operator="metricGovernance", name=None, id=None, extra=None, page_size=10, page_num=1):
    """
    指标操作

    :param operator: 操作类型，如 metricGovernance
    :param extra: 额外参数，格式为 key=value,key2=value2
    :param name: 指标名称
    :param id: 指标 ID

    """
    assets = []
    extra_dict = {}
    if extra is not None:
        # 解析 url 参数为 dict
        extra_dict = parse_url_params(extra)
    if name:
        req = list_assets_request("metrics", prefix=name, pageSize=page_size, pageNo=page_num)
        resp = request_get(req["path"], req["query_params"], req["custom_headers"]).json()
    elif id:
        metric_info = get_asset_by_id_request(asset_type="metrics", asset_id=id)
        resp = request_get(metric_info["path"], metric_info["query_params"], metric_info["custom_headers"]).json()
    elif extra_dict:
        req = list_assets_request("metrics", pageSize=page_size, pageNo=page_num, **extra_dict)
        resp = request_get(req["path"], req["query_params"], req["custom_headers"]).json()
    else:
        req = list_assets_request("metrics", pageSize=page_size, pageNo=page_num)
        resp = request_get(req["path"], req["query_params"], req["custom_headers"]).json()
    if resp:
        if "items" in resp:
            assets += resp["items"]
        else:
            assets.append(resp)

    if operator == "metricGovernance":
        _metricsGov(assets)


def _metricsGov(assets):
    """
    指标治理
    """
    prompt = """
你是一个指标治理专家，负责将未治理的指标进行治理，包括对指标名称进行汉化、添加指标描述、优化指标单位、分类、分组等
你要接受用户传来的指标信息，修改用户要修修改的字段后返回结果，保持数据结构一致性和完整性。接下来用户会传递json类型的数据，你要处理根据如下要求进行处理。返回数据时保持数据结构与传递时一致。不要包含任何其它非json字符。
  - 要求：
    1. 将指标名称(name)翻译为中文
    2. 添加指标描述(description)信息(描述信息要能够反映该指标的含义、以及指标能够反映出哪些信息等)
    3. 指标单位(unit)按照附表中《指标单位清单》进行填写
    4. 指标度量(measureType)按照指标意义选择，可选范围为[COUNTER,GAUGE,HISTOGRAM,SUMMARY]
    5. 指标类别(category)固定为"None",
    6. 指标性质(nature)可选：[Normal,Traffic,Error,Performance,Resource]，
    7. 所有者(owner)固定为空字符串，其它字段不做修改
  - 指标单位清单：none,short,percent,percentunit,humidity,dB,hex0x,hex,sci,locale,thousand,time,hertz,ns,µs,ms,s,m,h,d,dtdurationms,dtdurations,dthms,timeZh,hertzZh,nsZh,µsZh,msZh,sZh,mZh,hZh,dZh,dtdurationmsZh,dtdurationsZh,dthmsZh,data
    (IEC),bits,bytes,kbytes,mbytes,gbytes,data rate,pps,bps,Bps,Kbits,KBs,Mbits,MBs,Gbits,GBs,throughput,ops,reqps,rps,wps,iops,opm,rpm,wpm,length,lengthmm,lengthm,lengthft,lengthkm,lengthmi,area,areaM2,areaF2,areaMI2,mass,massmg,massg,masskg,masst,velocity,velocityms,velocitykmh,velocitymph,velocityknot,volume,mlitre,litre,m3,Nm3,dm3,gallons,energy,watt,kwatt,mwatt,Wm2,voltamp,kvoltamp,voltampreact,kvoltampreact,watth,kwatth,kwattm,joule,ev,amp,kamp,mamp,volt,kvolt,mvolt,dBm,ohm,lumens,temperature,celsius,farenheit,kelvin,pressure,pressurembar,pressurebar,pressurekbar,pressurehpa,pressurekpa,pressurehg,pressurepsi,force,forceNm,forcekNm,forceN,forcekN,flow,flowgpm,flowcms,flowcfs,flowcfm,litreh,flowlpm,flowmlpm,angle,degree,radian,grad,acceleration,accMS2,accFS2,accG,radiation,radbq,radci,radgy,radrad,radsv,radrem,radexpckg,radr,radsvh,concentration,ppm,conppb,conngm3,conμgm3,conmgm3,congm3
    """
    client = AIClient(system_prompt=prompt)
    
    if not assets:
        console.print("请指定指标名称或 ID", style="red")
        return
    
    # 使用rich进度条显示处理进度
    with Progress() as progress:
        task = progress.add_task("[green]处理指标治理...", total=len(assets))
        
        for i, asset in enumerate(assets):
            # 更新进度条描述，显示当前处理的指标
            asset_name = asset.get('name', f'指标{i+1}')
            progress.update(task, description=f"[green]正在处理: {asset_name}（{i+1}/{len(assets)}）")
            
            # console.print(asset, overflow="fold")
            asset_id = asset.pop("id")
            description = asset.get("description", "")
            if description:
                progress.advance(task)
                continue

            # 处理标签
            _labels = asset.pop("labels", {})
            labels = [x['key'] for x in _labels]
            labelList = copy.deepcopy(_labels)

            # 处理分组
            groupIds = [x.get('id') for x in asset.pop('groups', []) if 'id' in x]
            asset.update({"labelList": labelList, "labels": labels, 'groupIds': groupIds})

            resp = client.chat(json.dumps(asset))
            try:
                asset = json.loads(resp.content)
            except json.JSONDecodeError:
                console.print(f"[red]解析AI响应失败: {resp}[/red]")
                progress.advance(task)
                continue
            
            req_info = update_asset_request(asset_type="metrics", id=asset_id, data=asset)
            resp = request(req_info["method"], req_info["path"], data=req_info['data']).json()
            # 推进进度条
            progress.advance(task)


@command
def ai_chat(message, model=None, no_stream=False, no_markdown=False):
    """与AI大模型进行对话
    
    :param message: 要发送给AI的消息
    :param -m, --model: 指定使用的模型名称
    :param --no-stream: 是否使用流式输出
    :param --no-markdown: 禁用Markdown渲染，使用纯文本输出
    """
    try:
        client = AIClient(model_name=model)
        markdown = not no_markdown  # 转换参数逻辑
        
        if not no_stream:
            console.print(f"[bold green]AI ({client.get_current_model()}):[/bold green]")
            
            if markdown:
                # 使用Rich Live实现真正的流式Markdown渲染
                content_chunks = []
                
                with Live(Markdown(""), console=console, refresh_per_second=10) as live:
                    for chunk in client.stream_chat(message):
                        content_chunks.append(chunk)
                        # 实时更新Markdown渲染
                        current_content = "".join(content_chunks)
                        if current_content.strip():
                            live.update(Markdown(current_content))
                
                # 确保最终内容完整显示
                final_content = "".join(content_chunks)
                if not final_content.strip():
                    console.print("[yellow]AI返回了空响应[/yellow]")
            else:
                # 纯文本流式输出
                for chunk in client.stream_chat(message):
                    console.print(chunk, end="")
                console.print()  # 换行
        else:
            response = client.chat(message)
            console.print(f"[bold green]AI ({response.model}):[/bold green]")
            
            if markdown and response.content.strip():
                # 使用Markdown渲染
                markdown_content = Markdown(response.content)
                console.print(markdown_content)
            else:
                # 纯文本输出
                console.print(response.content)
            
            if response.usage:
                console.print(f"[dim]Token使用: {response.usage}[/dim]")
                
    except Exception as e:
        console.print(f"[red]AI请求失败: {e}[/red]")


@command
@arg("action", type=str, help="操作类型 (list|add|remove|set-default)", default="list")
def ai_config(action, model_name=None, endpoint=None, api_key=None):
    """管理AI模型配置
    
    :param action: 操作类型 (list|add|remove|set-default)
    :param -m, --model-name: 模型名称
    :param -e, --endpoint: API端点地址
    :param -k, --api-key: API密钥
    """
    try:
        # 对于add操作，允许空配置；其他操作需要有效配置
        allow_empty = (action == "add")
        config = AIConfig(allow_empty=allow_empty)
        
        if action == "list":
            models = config.list_models()
            default_model = config.get_default_model()
            
            console.print("[bold]可用的AI模型:[/bold]")
            for model in models:
                marker = " [green](默认)[/green]" if model == default_model else ""
                console.print(f"  • {model}{marker}")
                
        elif action == "add":
            if not all([model_name, endpoint, api_key]):
                console.print("[red]添加模型需要提供 --model-name, --endpoint 和 --api-key 参数[/red]")
                return
                
            from ketacli.sdk.ai.config import AIModelConfig
            model_config = AIModelConfig(
                name=model_name,
                endpoint=endpoint,
                api_key=api_key,
                model=model_name
            )
            config.add_model(model_config)
            console.print(f"[green]已添加模型: {model_name}[/green]")
            
        elif action == "remove":
            if not model_name:
                console.print("[red]删除模型需要提供 --model-name 参数[/red]")
                return
                
            config.remove_model(model_name)
            console.print(f"[green]已删除模型: {model_name}[/green]")
            
        elif action == "set-default":
            if not model_name:
                console.print("[red]设置默认模型需要提供 --model-name 参数[/red]")
                return
                
            config.set_default_model(model_name)
            console.print(f"[green]已设置默认模型: {model_name}[/green]")
            
        else:
            console.print("[red]无效的操作类型，支持: list, add, remove, set-default[/red]")
            
    except Exception as e:
        console.print(f"[red]配置操作失败: {e}[/red]")


def start():
    # 确保main被正确初始化
    import argcomplete
    argcomplete.autocomplete(main.parser)
    try:
        argcomplete.autocomplete(main.parser)
        main()
    except Exception:
        console.print_exception()


if __name__ == "__main__":
    start()



"""
AI大模型请求模块

提供统一的AI大模型调用接口，支持配置文件管理和回调验证。
"""

from .client import AIClient
from .config import AIConfig
from .validators import ResponseValidator

__all__ = ['AIClient', 'AIConfig', 'ResponseValidator']